"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IpcBusBridgeImpl = void 0;
const universal_1 = require("@electron-common-ipc/universal");
const IpcBusBrokerBridge_1 = require("./IpcBusBrokerBridge");
const IpcBusMainBridge_1 = require("./IpcBusMainBridge");
const IpcBusQueryState_collector_1 = require("./IpcBusQueryState-collector");
const IpcBusRendererBridge_1 = require("./IpcBusRendererBridge");
const IpcBusSocketBridge_1 = require("./IpcBusSocketBridge");
const IpcBusConnectorSocket_1 = require("../node/IpcBusConnectorSocket");
const utils_1 = require("../utils");
const IpcBusCommand_helpers_1 = require("../utils/IpcBusCommand-helpers");
class IpcBusBridgeImpl {
    constructor(contextType, _uuid, _stamp, _logger) {
        this._uuid = _uuid;
        this._stamp = _stamp;
        this._logger = _logger;
        const mainConnector = new IpcBusMainBridge_1.IpcBusBridgeConnectorMain(this._uuid, contextType, this);
        this._mainTransport = new IpcBusMainBridge_1.IpcBusBridgeTransportMain(mainConnector, this._uuid, this._stamp, this._logger);
        this._rendererConnector = new IpcBusRendererBridge_1.IpcBusRendererBridge(contextType, this);
        this._serializeMessage = new IpcBusCommand_helpers_1.SerializeMessage();
        this._queryStateManager = new IpcBusQueryState_collector_1.IpcBusQueryStateManager(this);
    }
    get mainTransport() {
        return this._mainTransport;
    }
    get rendererTransport() {
        return this._rendererConnector;
    }
    get socketTransport() {
        return this._socketTransport;
    }
    getWindowTarget(window, frameId) {
        return this._rendererConnector.getWindowTarget(window, frameId);
    }
    connect(arg1, arg2, arg3) {
        const options = (0, universal_1.CheckConnectOptions)(arg1, arg2, arg3);
        return this._rendererConnector.broadcastConnect(options).then(() => {
            if (this._socketTransport === undefined) {
                if (options.port || options.path) {
                    if (options.server === true) {
                        this._socketTransport = new IpcBusBrokerBridge_1.IpcBusBrokerBridge(6, this, this._logger);
                    }
                    else {
                        const connector = new IpcBusConnectorSocket_1.IpcBusConnectorSocket(this._uuid, 6);
                        this._socketTransport = new IpcBusSocketBridge_1.IpcBusTransportSocketBridge(connector, this, this._uuid, this._stamp, this._logger);
                    }
                    return this._socketTransport.broadcastConnect(options).catch(() => {
                        this._socketTransport = undefined;
                    });
                }
            }
            else if (!options.port && !options.path) {
                const socketTransport = this._socketTransport;
                this._socketTransport = undefined;
                return socketTransport.broadcastClose();
            }
            return Promise.resolve();
        });
    }
    close(_options) {
        return this._rendererConnector.broadcastClose().then(() => {
            if (this._socketTransport) {
                const socketTransport = this._socketTransport;
                this._socketTransport = undefined;
                return socketTransport.broadcastClose();
            }
            return Promise.resolve();
        });
    }
    startQueryState() {
        this._queryStateManager.start();
    }
    getQueryState() {
        return this._queryStateManager.processes;
    }
    getChannels() {
        const rendererChannels = this._rendererConnector.getChannels();
        const mainChannels = this._mainTransport.getChannels();
        return rendererChannels.concat(mainChannels);
    }
    _onBridgeChannelChanged(ipcCommand) {
        if (this._socketTransport) {
            switch (ipcCommand.kind) {
                case "LICA":
                    ipcCommand.kind = "BILICA";
                    break;
                case "LICR":
                    ipcCommand.kind = "BILICR";
                    break;
                default:
                    throw new Error(`Invalid command for bridge channel change event: ${ipcCommand.kind}`);
            }
            this._socketTransport.broadcastCommand(ipcCommand);
        }
    }
    _onRendererCommandReceived(ipcCommand) {
        switch (ipcCommand.kind) {
            case "QUSTR": {
                const queryStateResponse = ipcCommand.data;
                this._queryStateManager.collect(queryStateResponse);
                break;
            }
        }
    }
    _onRendererLogReceived(_ipcMessage, _data) { }
    _onRendererMessageReceived(ipcMessage, data, messagePorts) {
        if (ipcMessage.isRawData) {
            const rawData = data;
            const packetCore = (0, utils_1.fixRawData)(rawData);
            if (this._mainTransport.onConnectorPacketReceived(ipcMessage, packetCore, messagePorts) === false) {
                const hasSocketChannel = this._socketTransport && this._socketTransport.isTarget(ipcMessage);
                if (hasSocketChannel) {
                    this._socketTransport.broadcastData(ipcMessage, rawData, messagePorts);
                }
            }
        }
        else {
            const args = data;
            if (this._mainTransport.onConnectorArgsReceived(ipcMessage, args, messagePorts) === false) {
                const hasSocketChannel = this._socketTransport && this._socketTransport.isTarget(ipcMessage);
                if (hasSocketChannel) {
                    const packet = this._serializeMessage.serialize(ipcMessage, args);
                    this._socketTransport.broadcastPacket(ipcMessage, packet);
                }
            }
        }
    }
    _onMainCommandReceived(ipcCommand) {
        switch (ipcCommand.kind) {
            case "QUSTR": {
                const queryStateResponse = ipcCommand.data;
                this._queryStateManager.collect(queryStateResponse);
                break;
            }
        }
    }
    _onMainLogReceived(_ipcMessage, _args) { }
    _onMainMessageReceived(ipcMessage, args, messagePorts) {
        if (this._rendererConnector.broadcastData(ipcMessage, args, messagePorts) === false) {
            const hasSocketChannel = this._socketTransport && this._socketTransport.isTarget(ipcMessage);
            if (hasSocketChannel) {
                if (ipcMessage.isRawData) {
                    this._socketTransport.broadcastData(ipcMessage, args);
                }
                else {
                    const packet = this._serializeMessage.serialize(ipcMessage, args);
                    this._socketTransport.broadcastPacket(ipcMessage, packet);
                }
            }
        }
    }
    _onSocketCommandReceived(ipcCommand) {
        switch (ipcCommand.kind) {
            case "QUSTR": {
                const queryStateResponse = ipcCommand.data;
                this._queryStateManager.collect(queryStateResponse);
                break;
            }
        }
    }
    _onSocketLogReceived(_ipcMessage, _ipcPacketBufferCore) { }
    _onSocketMessageReceived(ipcMessage, ipcPacketBufferCore) {
        if (this._mainTransport.onMessageReceived(false, ipcMessage, undefined, ipcPacketBufferCore) === false) {
            return this._rendererConnector.broadcastPacket(ipcMessage, ipcPacketBufferCore);
        }
        return true;
    }
    _onSocketRequestResponseReceived(ipcResponse, ipcPacketBufferCore) {
        if (this._mainTransport.onRequestResponseReceived(false, ipcResponse, undefined, ipcPacketBufferCore) === false) {
            return this._rendererConnector.broadcastPacket(ipcResponse, ipcPacketBufferCore);
        }
        return true;
    }
    _onSocketClosed() {
        this._socketTransport = undefined;
    }
}
exports.IpcBusBridgeImpl = IpcBusBridgeImpl;
//# sourceMappingURL=IpcBusBridgeImpl.js.map