"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IpcBusRendererBridge = void 0;
const universal_1 = require("@electron-common-ipc/universal");
const IpcBusLog_factory_main_1 = require("../log/IpcBusLog-factory-main");
const IpcBusConnectorRenderer_1 = require("../renderer/IpcBusConnectorRenderer");
const utils_1 = require("../utils");
const IpcBusCommand_helpers_1 = require("../utils/IpcBusCommand-helpers");
const electron = (0, utils_1.requireElectron)();
class IpcBusRendererBridge {
    constructor(contextType, bridge) {
        this._contextType = contextType;
        this._bridge = bridge;
        this._ipcMain = electron.ipcMain;
        const electronVersion = process.versions.electron.split('.')[0];
        this._earlyIPCIssueFixed = Number(electronVersion) >= 12;
        this._subscriptions = new universal_1.ChannelConnectionMap('IPCBus:RendererBridge');
        this._endpoints = new Map();
        const bridgeMockPeer = { id: 'renderer-bridge', type: this._contextType };
        this._subscriptions.client = {
            channelAdded: (channel) => {
                const ipcCommand = {
                    kind: "LICA",
                    peer: bridgeMockPeer,
                    channel,
                };
                this._bridge._onBridgeChannelChanged(ipcCommand);
            },
            channelRemoved: (channel) => {
                const ipcCommand = {
                    kind: "LICR",
                    peer: bridgeMockPeer,
                    channel,
                };
                this._bridge._onBridgeChannelChanged(ipcCommand);
            },
        };
        this._onHandshakeReceived = this._onHandshakeReceived.bind(this);
        this._onPortMessageReceived = this._onPortMessageReceived.bind(this);
        this._onPortCommandReceived = this._onPortCommandReceived.bind(this);
        this._onIPCCommandReceived = this._onIPCCommandReceived.bind(this);
        this._onIPCMessageReceived = this._onIPCMessageReceived.bind(this);
        this._onIPCLogReceived = this._onIPCLogReceived.bind(this);
    }
    getWindowTarget(window, frameId) {
        let result;
        if (frameId === undefined) {
            for (const endpoint of this._endpoints.values()) {
                if (endpoint.process.wcid === window.webContents.id && endpoint.process.isMainFrame) {
                    result = endpoint;
                    break;
                }
            }
        }
        else {
            for (const endpoint of this._endpoints.values()) {
                if (endpoint.process.wcid === window.webContents.id && endpoint.process.frameid === frameId) {
                    result = endpoint;
                    break;
                }
            }
        }
        return result;
    }
    isTarget(ipcMessage) {
        if (this._subscriptions.hasChannel(ipcMessage.channel)) {
            return true;
        }
        return (0, IpcBusCommand_helpers_1.GetTargetRenderer)(ipcMessage) !== undefined;
    }
    getChannels() {
        return this._subscriptions.getChannels();
    }
    broadcastConnect(_options) {
        this._ipcMain.removeListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_HANDSHAKE, this._onHandshakeReceived);
        this._ipcMain.addListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_HANDSHAKE, this._onHandshakeReceived);
        this._ipcMain.removeListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_COMMAND, this._onIPCCommandReceived);
        this._ipcMain.addListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_COMMAND, this._onIPCCommandReceived);
        this._ipcMain.removeListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_MESSAGE, this._onIPCMessageReceived);
        this._ipcMain.addListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_MESSAGE, this._onIPCMessageReceived);
        this._ipcMain.removeListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_LOGROUNDTRIP, this._onIPCLogReceived);
        this._ipcMain.addListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_LOGROUNDTRIP, this._onIPCLogReceived);
        return Promise.resolve();
    }
    broadcastClose(_options) {
        this._ipcMain.removeListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_HANDSHAKE, this._onHandshakeReceived);
        this._ipcMain.removeListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_COMMAND, this._onIPCCommandReceived);
        this._ipcMain.removeListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_MESSAGE, this._onIPCMessageReceived);
        this._ipcMain.removeListener(IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_LOGROUNDTRIP, this._onIPCLogReceived);
        return Promise.resolve();
    }
    _onHandshakeReceived(event, ipcCommand) {
        const logger = (0, IpcBusLog_factory_main_1.CreateIpcBusLog)();
        const webContents = event.sender;
        const realPeer = ipcCommand.peer;
        realPeer.process.wcid = webContents.id;
        realPeer.process.frameid = event.frameId;
        try {
            realPeer.process.rid = event.processId || webContents.getProcessId();
        }
        catch (err) {
            realPeer.process.rid = -1;
        }
        try {
            realPeer.process.pid = webContents.getOSProcessId();
        }
        catch (err) {
            realPeer.process.pid = webContents.id;
        }
        this._trackRendererDestruction(webContents);
        const peerCopy = Object.assign({}, realPeer);
        const endpoint = Object.assign(realPeer, { webContents });
        const key = (0, IpcBusCommand_helpers_1.CreateKeyForEndpoint)(realPeer);
        this._endpoints.set(key, endpoint);
        if (event.ports) {
            endpoint.messagePort = event.ports[0];
            endpoint.messagePort.on('close', () => {
                this._onEndpointShutdown(ipcCommand);
            });
            endpoint.messagePort.addListener('message', this._onPortMessageReceived);
            endpoint.messagePort.start();
            endpoint.commandPort = event.ports[1];
            if (endpoint.commandPort) {
                endpoint.commandPort.on('close', () => {
                    this._onEndpointShutdown(ipcCommand);
                });
                endpoint.commandPort.addListener('message', this._onPortCommandReceived);
                endpoint.commandPort.start();
            }
        }
        const handshake = {
            peer: peerCopy,
            logLevel: logger.level,
        };
        const frameTarget = [event.processId, event.frameId];
        if (webContents.getURL()) {
            if (this._earlyIPCIssueFixed) {
                webContents.sendToFrame(frameTarget, IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_HANDSHAKE, handshake);
                return;
            }
            if (!webContents.isLoadingMainFrame()) {
                webContents.sendToFrame(frameTarget, IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_HANDSHAKE, handshake);
                return;
            }
        }
        webContents.on('did-finish-load', () => {
            webContents.sendToFrame(frameTarget, IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_HANDSHAKE, handshake);
        });
    }
    _trackRendererDestruction(webContents) {
        const webContentsId = webContents.id;
        webContents.once('destroyed', () => {
            const entries = this._subscriptions.getConns().filter((entry) => {
                return entry.data.process.wcid === webContentsId;
            });
            for (let i = 0, l = entries.length; i < l; ++i) {
                this.deleteEndpointKey(entries[i].key);
            }
        });
    }
    deleteEndpointKey(key) {
        const endpoint = this._endpoints.get(key);
        if (endpoint) {
            this._endpoints.delete(key);
            this._subscriptions.remove(key);
            if (endpoint.messagePort) {
                endpoint.messagePort.close();
                endpoint.messagePort.removeListener('message', this._onPortMessageReceived);
                endpoint.messagePort = undefined;
            }
            if (endpoint.commandPort) {
                endpoint.commandPort.close();
                endpoint.commandPort.removeListener('message', this._onPortCommandReceived);
                endpoint.commandPort = undefined;
            }
        }
    }
    _onEndpointHandshake(_ipcCommand) { }
    _onEndpointShutdown(ipcCommand) {
        var _a;
        const peers = (_a = ipcCommand.peers) !== null && _a !== void 0 ? _a : [ipcCommand.peer];
        peers.forEach((peer) => {
            const key = (0, IpcBusCommand_helpers_1.CreateKeyForEndpoint)(peer);
            this.deleteEndpointKey(key);
        });
    }
    broadcastCommand(ipcCommand) {
        this._endpoints.forEach((endpoint) => {
            const frameTarget = [endpoint.process.rid, endpoint.process.frameid];
            endpoint.webContents.sendToFrame(frameTarget, IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_COMMAND, ipcCommand);
        });
    }
    broadcastBuffers(_ipcMessage, _buffers) {
        throw 'not implemented';
    }
    broadcastPacket(ipcMessage, ipcPacketBufferCore) {
        const rawData = ipcPacketBufferCore.getRawData();
        return this._broadcastData(false, ipcMessage, rawData);
    }
    broadcastData(ipcMessage, data, messagePorts) {
        return this._broadcastData(false, ipcMessage, data, messagePorts);
    }
    _broadcastData(local, ipcMessage, data, messagePorts) {
        const target = (0, IpcBusCommand_helpers_1.GetTargetRenderer)(ipcMessage);
        if (target) {
            const key = (0, IpcBusCommand_helpers_1.CreateKeyForEndpoint)(target);
            const endpoint = this._endpoints.get(key);
            if (endpoint) {
                if (!messagePorts) {
                    const frameTarget = [endpoint.process.rid, endpoint.process.frameid];
                    endpoint.webContents.sendToFrame(frameTarget, IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_MESSAGE, ipcMessage, data);
                }
                else {
                    endpoint.messagePort.postMessage([ipcMessage, data], messagePorts);
                }
            }
            return true;
        }
        if (ipcMessage.kind === "MES") {
            const channelConns = this._subscriptions.getChannelConns(ipcMessage.channel);
            if (channelConns) {
                const key = local ? (0, IpcBusCommand_helpers_1.CreateKeyForEndpoint)(ipcMessage.peer) : -1;
                if (!messagePorts) {
                    channelConns.forEach((entry) => {
                        if (entry.key !== key) {
                            const frameTarget = [entry.data.process.rid, entry.data.process.frameid];
                            entry.data.webContents.sendToFrame(frameTarget, IpcBusConnectorRenderer_1.IPCBUS_TRANSPORT_RENDERER_MESSAGE, ipcMessage, data);
                        }
                    });
                }
                else {
                    channelConns.forEach((entry) => {
                        if (entry.key !== key) {
                            entry.data.messagePort.postMessage([ipcMessage, data], messagePorts);
                        }
                    });
                }
            }
        }
        return false;
    }
    _onIPCLogReceived(_event, ipcMessage, args) {
        this._bridge._onRendererLogReceived(ipcMessage, args);
    }
    _onIPCCommandReceived(_event, ipcCommand) {
        switch (ipcCommand.kind) {
            case "HAN":
                this._onEndpointHandshake(ipcCommand);
                return true;
            case "SHT":
                this._onEndpointShutdown(ipcCommand);
                return true;
            case "LICA":
                this._addListeners(ipcCommand);
                return true;
            case "LICR":
                this._removeListeners(ipcCommand, 'release');
                return true;
            case "LICRA":
                this._removeListeners(ipcCommand, 'all');
                return true;
            case "LIR": {
                this._removeListeners(ipcCommand, 'remove');
                return true;
            }
            case "QUST":
            case "QUSTR":
                this._bridge._onRendererCommandReceived(ipcCommand);
                return true;
        }
        return false;
    }
    _addListeners(ipcCommand) {
        var _a;
        const peers = (_a = ipcCommand.peers) !== null && _a !== void 0 ? _a : [ipcCommand.peer];
        peers.forEach((peer) => {
            const key = (0, IpcBusCommand_helpers_1.CreateKeyForEndpoint)(peer);
            const endpointWC = this._endpoints.get(key);
            if (endpointWC) {
                this._subscriptions.addRef(ipcCommand.channel, key, endpointWC);
            }
        });
    }
    _removeListeners(ipcCommand, type) {
        var _a;
        const peers = (_a = ipcCommand.peers) !== null && _a !== void 0 ? _a : [ipcCommand.peer];
        peers.forEach((peer) => {
            const key = (0, IpcBusCommand_helpers_1.CreateKeyForEndpoint)(peer);
            switch (type) {
                case 'release':
                    this._subscriptions.release(ipcCommand.channel, key);
                    break;
                case 'all':
                    this._subscriptions.releaseAll(ipcCommand.channel, key);
                    break;
                case 'remove':
                    this._subscriptions.remove(key);
                    break;
            }
        });
    }
    _onPortCommandReceived(event) {
        return this._onIPCCommandReceived(undefined, event.data);
    }
    _onIPCMessageReceived(_event, ipcMessage, data) {
        if (this._broadcastData(true, ipcMessage, data) === false) {
            this._bridge._onRendererMessageReceived(ipcMessage, data);
        }
    }
    _onPortMessageReceived(event) {
        const [ipcMessage, data] = event.data;
        if (this._broadcastData(true, ipcMessage, data, event.ports) === false) {
            this._bridge._onRendererMessageReceived(ipcMessage, data, event.ports);
        }
    }
    queryState() {
        const peersJSON = {};
        const processChannelsJSON = {};
        const channels = this._subscriptions.getChannels();
        for (let i = 0; i < channels.length; ++i) {
            const channel = channels[i];
            const processChannelJSON = (processChannelsJSON[channel] = {
                name: channel,
                refCount: 0,
            });
            const channelConns = this._subscriptions.getChannelConns(channel);
            channelConns.forEach((clientRef) => {
                processChannelJSON.refCount += clientRef.refCount;
                const endpoint = clientRef.data;
                const processID = (0, utils_1.createProcessID)(endpoint, endpoint.process);
                const peerJSON = (peersJSON[processID] = peersJSON[processID] || {
                    peer: endpoint,
                    channels: {},
                });
                const peerChannelJSON = (peerJSON.channels[channel] = peerJSON.channels[channel] || {
                    name: channel,
                    refCount: 0,
                });
                peerChannelJSON.refCount += clientRef.refCount;
            });
        }
        const results = {
            type: 'renderer-bridge',
            contextId: (0, universal_1.createContextId)(this._contextType),
            channels: processChannelsJSON,
            peers: peersJSON,
        };
        return results;
    }
}
exports.IpcBusRendererBridge = IpcBusRendererBridge;
//# sourceMappingURL=IpcBusRendererBridge.js.map