"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IpcBusConnectorSocket = void 0;
const universal_1 = require("@electron-common-ipc/universal");
const json_helpers_1 = require("json-helpers");
const net = require("net");
const socket_serializer_1 = require("socket-serializer");
const utils_1 = require("../utils");
const IpcBusCommand_helpers_1 = require("../utils/IpcBusCommand-helpers");
class IpcBusConnectorSocket extends universal_1.IpcBusConnectorImpl {
    constructor(uuid, contextType, logger) {
        super(uuid, contextType, 'connector-socket', logger);
        this._localBinds = {};
        this._bufferListReader = new socket_serializer_1.BufferListReader();
        this._packetIn = new socket_serializer_1.IpcPacketBufferList();
        this._packetIn.JSON = json_helpers_1.JSONParserV1;
        this._serializeMessage = new IpcBusCommand_helpers_1.SerializeMessage();
        this._netBinds = {};
        this._netBinds['error'] = this._onSocketError.bind(this);
        this._netBinds['close'] = this._onSocketClose.bind(this);
        this._netBinds['data'] = this._onSocketData.bind(this);
        this._netBinds['end'] = this._onSocketEnd.bind(this);
    }
    get socket() {
        return this._socket;
    }
    _onSocketError(err) {
        var _a;
        const msg = `[IPCBusTransport:Net ${this.id}] socket error ${err}`;
        (_a = this._logger) === null || _a === void 0 ? void 0 : _a.error(msg);
        this.onConnectorShutdown();
        this._reset(false);
    }
    _onSocketClose() {
        var _a;
        const msg = `[IPCBusTransport:Net ${this.id}] socket close`;
        (_a = this._logger) === null || _a === void 0 ? void 0 : _a.info(msg);
        this.onConnectorShutdown();
        this._reset(false);
    }
    _onSocketEnd() {
        var _a;
        const msg = `[IPCBusTransport:Net ${this.id}] socket end`;
        (_a = this._logger) === null || _a === void 0 ? void 0 : _a.info(msg);
        this.onConnectorShutdown();
        this._reset(false);
    }
    _onSocketData(buffer) {
        this._bufferListReader.appendBuffer(buffer);
        if (this._packetIn.decodeFromReader(this._bufferListReader)) {
            do {
                const ipcCommandBase = this._packetIn.parseArrayAt(0);
                switch (ipcCommandBase.kind) {
                    case "MES":
                        this._client.onMessageReceived(false, ipcCommandBase, undefined, this._packetIn);
                        break;
                    case "RQR":
                        this._client.onRequestResponseReceived(false, ipcCommandBase, undefined, this._packetIn);
                        break;
                    case "LOGRT":
                        this._client.onLogReceived(ipcCommandBase, undefined, this._packetIn);
                        break;
                    default:
                        this._client.onConnectorCommandBase(ipcCommandBase);
                        break;
                }
            } while (this._packetIn.decodeFromReader(this._bufferListReader));
            this._bufferListReader.reduce();
        }
    }
    _reset(endSocket) {
        this._socketWriter = undefined;
        if (this._socket) {
            const socket = this._socket;
            this._socket = undefined;
            for (const key in this._netBinds) {
                socket.removeListener(key, this._netBinds[key]);
            }
            if (endSocket) {
                socket.end();
            }
        }
    }
    handshakeInternal(client, peer, options) {
        return new Promise((resolve, reject) => {
            options = (0, universal_1.CheckConnectOptions)(options);
            if (!options.port && !options.path) {
                reject('Connection options not provided');
                return;
            }
            this._socketBuffer = options.socketBuffer;
            let timer = undefined;
            let fctReject = () => { };
            if (options.timeoutDelay >= 0) {
                timer = setTimeout(() => {
                    timer = undefined;
                    const msg = `[IPCBusTransport:Net ${this.id}] error = timeout (${options.timeoutDelay} ms) on ${JSON.stringify(options)}`;
                    fctReject(msg);
                }, options.timeoutDelay);
            }
            const catchError = (err) => {
                const msg = `[IPCBusTransport:Net ${this.id}] socket error = ${err} on ${JSON.stringify(options)}`;
                fctReject(msg);
            };
            const catchClose = () => {
                const msg = `[IPCBusTransport:Net ${this.id}] socket close`;
                fctReject(msg);
            };
            const socket = new net.Socket();
            socket.unref();
            const catchConnect = () => {
                var _a;
                clearTimeout(timer);
                this.addClient(client);
                this._removeLocalBinds();
                this._socket = socket;
                for (const key in this._netBinds) {
                    this._socket.addListener(key, this._netBinds[key]);
                }
                (_a = this._logger) === null || _a === void 0 ? void 0 : _a.info(`[IPCBusTransport:Net ${this.id}] connected on ${JSON.stringify(options)}`);
                if (!this._socketBuffer) {
                    this._socketWriter = new socket_serializer_1.SocketWriter(this._socket);
                }
                else if (this._socketBuffer < 0) {
                    this._socketWriter = new socket_serializer_1.DelayedSocketWriter(this._socket);
                }
                else if (this._socketBuffer > 0) {
                    this._socketWriter = new socket_serializer_1.BufferedSocketWriter(this._socket, this._socketBuffer);
                }
                const handshake = {
                    peer,
                };
                resolve(handshake);
            };
            fctReject = (msg) => {
                var _a;
                clearTimeout(timer);
                this._removeLocalBinds();
                this._reset(false);
                (_a = this._logger) === null || _a === void 0 ? void 0 : _a.error(msg);
                reject(msg);
            };
            this._localBinds['error'] = catchError.bind(this);
            this._localBinds['close'] = catchClose.bind(this);
            this._localBinds['connect'] = catchConnect.bind(this);
            for (const key in this._localBinds) {
                socket.addListener(key, this._localBinds[key]);
            }
            if (options.path) {
                socket.connect(options.path);
            }
            else if (options.port && options.host) {
                socket.connect(options.port, options.host);
            }
            else {
                socket.connect(options.port);
            }
        });
    }
    shutdownInternal(options) {
        options = options || {};
        if (options.timeoutDelay === undefined) {
            options.timeoutDelay = utils_1.IPC_BUS_TIMEOUT;
        }
        this._removeLocalBinds();
        return new Promise((resolve) => {
            if (this._socket) {
                let timer;
                const socket = this._socket;
                const catchClose = () => {
                    clearTimeout(timer);
                    for (const key in this._localBinds) {
                        socket.removeListener(key, this._localBinds[key]);
                    }
                    this.onConnectorShutdown();
                    socket.destroy();
                    resolve();
                };
                this._localBinds['close'] = catchClose.bind(this);
                for (const key in this._localBinds) {
                    socket.addListener(key, this._localBinds[key]);
                }
                this._reset(true);
            }
            else {
                resolve();
            }
        });
    }
    postMessage(ipcMessage, args) {
        if (this._socketWriter) {
            this._serializeMessage.writeMessage(this._socketWriter, ipcMessage, args);
        }
    }
    postCommand(ipcCommand) {
        if (this._socketWriter) {
            this._serializeMessage.writeCommand(this._socketWriter, ipcCommand);
        }
    }
    _removeLocalBinds() {
        var _a;
        for (const key in this._localBinds) {
            (_a = this._socket) === null || _a === void 0 ? void 0 : _a.removeListener(key, this._localBinds[key]);
        }
        this._localBinds = {};
    }
}
exports.IpcBusConnectorSocket = IpcBusConnectorSocket;
//# sourceMappingURL=IpcBusConnectorSocket.js.map