"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SmartMessageBag = exports.SerializeMessage = exports.CreateTargetChannel = exports.CreateKeyForEndpoint = exports.GetTargetRenderer = exports.GetTargetProcess = exports.GetTargetMain = void 0;
require("@electron-common-ipc/universal");
const json_helpers_1 = require("json-helpers");
const socket_serializer_1 = require("socket-serializer");
const uuid_1 = require("./uuid");
const TargetSignature = `_target_`;
const TargetMainSignature = `${TargetSignature}main_`;
const TargetProcessSignature = `${TargetSignature}proc_`;
const TargetRendererSignature = `${TargetSignature}rend_`;
const TargetSignatureLength = TargetMainSignature.length;
const TargetSignatures = {
    [1]: TargetProcessSignature,
    [0]: TargetProcessSignature,
    [2]: TargetRendererSignature,
    [6]: TargetMainSignature,
};
function _GetTargetFromChannel(targetTypeSignature, ipcMessage) {
    if (ipcMessage.channel && ipcMessage.channel.lastIndexOf(TargetSignature, 0) === 0) {
        if (ipcMessage.channel.lastIndexOf(targetTypeSignature, 0) !== 0) {
            return undefined;
        }
        const index = ipcMessage.channel.indexOf(TargetSignature, TargetSignatureLength);
        return JSON.parse(ipcMessage.channel.substr(TargetSignatureLength, index - TargetSignatureLength));
    }
    return undefined;
}
function GetTargetMain(ipcMessage, checkChannel = false) {
    if (ipcMessage.target) {
        return ipcMessage.target.type === 6 ? ipcMessage.target : undefined;
    }
    if (checkChannel) {
        return _GetTargetFromChannel(TargetMainSignature, ipcMessage);
    }
    return undefined;
}
exports.GetTargetMain = GetTargetMain;
function GetTargetProcess(ipcMessage, checkChannel = false) {
    if (ipcMessage.target) {
        return ipcMessage.target.type === 1 || ipcMessage.target.type === 0
            ? ipcMessage.target
            : undefined;
    }
    if (checkChannel) {
        return _GetTargetFromChannel(TargetProcessSignature, ipcMessage);
    }
    return undefined;
}
exports.GetTargetProcess = GetTargetProcess;
function GetTargetRenderer(ipcMessage, checkChannel = false) {
    if (ipcMessage.target) {
        return ipcMessage.target.type === 2
            ? (ipcMessage.target)
            : undefined;
    }
    if (checkChannel) {
        return _GetTargetFromChannel(TargetRendererSignature, ipcMessage);
    }
    return undefined;
}
exports.GetTargetRenderer = GetTargetRenderer;
function CreateKeyForEndpoint(endpoint) {
    var _a, _b;
    if (((_a = endpoint.process) === null || _a === void 0 ? void 0 : _a.wcid) && ((_b = endpoint.process) === null || _b === void 0 ? void 0 : _b.frameid)) {
        return `wcid-${(endpoint.process.wcid << 8) + endpoint.process.frameid}`;
    }
    return endpoint.id;
}
exports.CreateKeyForEndpoint = CreateKeyForEndpoint;
function CreateTargetChannel(peer) {
    const targetTypeSignature = TargetSignatures[peer.type] || `_no_target_`;
    return `${targetTypeSignature}${JSON.stringify(peer)}${TargetSignature}${(0, uuid_1.uuidProvider)()}`;
}
exports.CreateTargetChannel = CreateTargetChannel;
class SerializeMessage {
    constructor() {
        this._packetOut = new socket_serializer_1.IpcPacketBuffer();
        this._packetOut.JSON = json_helpers_1.JSONParserV1;
    }
    serialize(ipcMessage, args) {
        if (!ipcMessage.isRawData) {
            ipcMessage.isRawData = true;
            json_helpers_1.JSONParserV1.install();
            this._packetOut.serialize([ipcMessage, args]);
            json_helpers_1.JSONParserV1.uninstall();
            return this._packetOut;
        }
        return undefined;
    }
    writeMessage(writer, ipcMessage, args) {
        ipcMessage.isRawData = true;
        this._packetOut.write(writer, [ipcMessage, args]);
    }
    writeCommand(writer, ipcCommand) {
        this._packetOut.write(writer, [ipcCommand]);
    }
}
exports.SerializeMessage = SerializeMessage;
class SmartMessageBag {
    constructor() {
        this._packetOut = new socket_serializer_1.IpcPacketBuffer();
        this._packetOut.JSON = json_helpers_1.JSONParserV1;
    }
    set(ipcMessage, data) {
        this._ipcMessage = ipcMessage;
        this._supportStructureClone = undefined;
        if (this._ipcMessage.isRawData) {
            this._rawData = data;
            this._data = undefined;
        }
        else {
            this._data = data;
            this._rawData = undefined;
        }
    }
    serialize(ipcMessage, args) {
        if (!ipcMessage.isRawData) {
            ipcMessage.isRawData = true;
            json_helpers_1.JSONParserV1.install();
            this._packetOut.serialize([ipcMessage, args]);
            json_helpers_1.JSONParserV1.uninstall();
            return this._packetOut;
        }
        return undefined;
    }
    writeMessage(writer, ipcMessage, args) {
        if (this._rawData) {
            this._packetOut.write(writer, this._rawData);
        }
        else {
            ipcMessage.isRawData = true;
            this._packetOut.write(writer, [ipcMessage, args]);
            ipcMessage.isRawData = false;
        }
    }
    writeCommand(writer, ipcCommand) {
        this._packetOut.write(writer, [ipcCommand]);
    }
    sendIPCMessage(ipc, channel) {
        if (this._data) {
            if (this._supportStructureClone !== false) {
                try {
                    ipc.send(channel, this._ipcMessage, this._data);
                    this._supportStructureClone = true;
                    return;
                }
                catch (err) {
                    this._supportStructureClone = false;
                }
            }
        }
        this._ipcMessage.isRawData = true;
        if (!this._rawData) {
            json_helpers_1.JSONParserV1.install();
            this._packetOut.serialize([this._ipcMessage, this._data]);
            json_helpers_1.JSONParserV1.uninstall();
            this._rawData = this._packetOut.getRawData();
        }
        ipc.send(channel, this._ipcMessage, this._rawData);
        this._ipcMessage.isRawData = false;
    }
    sendIPCMessageTo(ipc, wcid, channel) {
        if (this._data) {
            if (this._supportStructureClone !== false) {
                try {
                    ipc.sendTo(wcid, channel, this._ipcMessage, this._data);
                    this._supportStructureClone = true;
                    return;
                }
                catch (err) {
                    this._supportStructureClone = false;
                }
            }
        }
        this._ipcMessage.isRawData = true;
        if (!this._rawData) {
            json_helpers_1.JSONParserV1.install();
            this._packetOut.serialize([this._ipcMessage, this._data]);
            json_helpers_1.JSONParserV1.uninstall();
            this._rawData = this._packetOut.getRawData();
        }
        ipc.sendTo(wcid, channel, this._ipcMessage, this._rawData);
        this._ipcMessage.isRawData = false;
    }
    sendPortMessage(port, messagePorts) {
        if (this._data) {
            if (this._supportStructureClone !== false) {
                try {
                    port.postMessage([this._ipcMessage, this._data], messagePorts);
                    this._supportStructureClone = true;
                    return;
                }
                catch (err) {
                    this._supportStructureClone = false;
                }
            }
        }
        this._ipcMessage.isRawData = true;
        if (!this._rawData) {
            json_helpers_1.JSONParserV1.install();
            this._packetOut.serialize([this._ipcMessage, this._data]);
            json_helpers_1.JSONParserV1.uninstall();
            this._rawData = this._packetOut.getRawData();
        }
        port.postMessage([this._ipcMessage, this._rawData], messagePorts);
        this._ipcMessage.isRawData = false;
    }
}
exports.SmartMessageBag = SmartMessageBag;
//# sourceMappingURL=IpcBusCommand-helpers.js.map